﻿#pragma TextEncoding	= "UTF-8"
#pragma rtGlobals 		= 3
#pragma IgorVersion		= 6.36
#pragma DefaultTab		= {3,20,4}		// Set default tab width in Igor Pro 9 and later

//________________________________________________________________________________________________
//	Written by Stephan Thuermer - https://www.wavemetrics.com/user/chozo
//
//	This is a collection of functions for graphing data.
//	Contents:
//		- Graph style macros
//		- QuickColorSpectrum :		color up to 12 traces with a set of fixed colors
//		- TraceOffsetPanel :		a small panel to offset and color traces
//		- SweepWaterfall :			generates a waterfall plot out of 2D data or a folder
//		- MakeGraphSizePanel :		a small panel to set the graph size and margins
//________________________________________________________________________________________________

Menu "Spectra Tools"
	"Waterfall Plot from 2D Data or Folder", /Q,	CreateBrowser; SweepWaterfall()
	"Quick Graph Size Panel .../F10", /Q,			MakeGraphSizePanel()
	"Quick Colorize Traces/F11", /Q,				QuickColorSpectrum()
	"Graph Style and Offset Panel .../F12", /Q,		TraceOffsetPanel()
End

Menu "GraphPopup"
	"Left: Autoscale Only Visible", /Q, SetAxis/A=2/E=0 left
	"Left: Autoscale From Zero", /Q, SetAxis/A=2/E=1 left
	"Traces: Undo All Scaling", /Q, ModifyGraph muloffset={0,0}
	"Traces: Undo All Offsets", /Q, ModifyGraph offset={0,0}
End

//________________________________________________________________________________________________
//	Creates waterfall plot from a sweep wave or a folder of waves
//________________________________________________________________________________________________

Function SweepWaterfall()
	Wave/Z inwave = $GetBrowserSelection(0)														// get the first selected wave
	if (!WaveExists(inwave))
		Print "Nothing selected"
		return -1
	endif
	String Datalist	= ""
	Variable OneD = WaveDims(inwave) == 1														// first check, if the incoming wave is 1D or 2D
	DFREF saveDFR = GetDataFolderDFR()
	if (OneD)
		SetDataFolder GetWavesDataFolderDFR(inwave)
		Datalist = WaveList("!Int_*", ";", "DIMS:1")
		Datalist = SortList(Datalist, ";", 16)
	endif
	
	Display/W=(230,50,650,450)/K=1/N=$UniqueName("WaterfallGraph", 6, 0) as "Waterfall of "+NameOfWave(inwave)
	Variable i, Items = OneD ? ItemsInList(Datalist) : DimSize(inwave,1)
	for (i = Items - 1; i > -1; i -=1)															// go backwards (in reverse order, so that the later traces are BEHIND the earlier ones)
		if (OneD)
			AppendToGraph/Q $Stringfromlist(i,Datalist)
		else
			AppendToGraph/Q inwave[][i]
		endif
	endfor
	SetDataFolder saveDFR
	Execute/Q "BoxStyle()"
	ColorSpectrumFromTable(0, "Rainbow")
	
	String Traces = TraceNameList("",";",1)
	Variable Offset	= WaveMax(inwave) * 0.05													// the initial distance will be set in percent from the maximal value of the selected wave
	for (i = 1; i < Items; i += 1)
		ModifyGraph offset($StringFromList(Items - 1 - i,Traces))={,i*Offset}
	endfor
End

//________________________________________________________________________________________________
//	change the color of traces
//________________________________________________________________________________________________

Function QuickColorSpectrum()																	// colors traces with 12 different colors (hard-coded)
	String Traces	= TraceNameList("",";",1)
	Variable Items	= ItemsInList(Traces)
	if (Items == 0)
		return 0
	endif
	Make/FREE/N=(11,3) colors = {{65280,0,0}, {65280,43520,0}, {0,65280,0}, {0,52224,0}, {0,65280,65280}, {0,43520,65280}, {0,15872,65280}, {65280,16384,55552}, {44253,29492,58982}, {26411,1,52428}, {26112,26112,26112}, {0,0,0}}
	Variable colorcount = DimSize(colors,1)
	Variable i, colori, leap = 1
	if (Items < colorcount/2)																	// introduce a leap value to choose more distant colors for only a few traces
		leap = trunc(colorcount/(Items-1))-1
	endif
	for (i = 0, colori = 0; i < Items; i += 1, colori += leap)
		colori = colori == colorcount? 0 : colori												// loop colors around
		ModifyGraph rgb($StringFromList(i,Traces))=(colors[0][colori],colors[1][colori],colors[2][colori])	// set new color offset
	endfor
End

//________________________________________________________________________________________________
//	Graph Macros
//________________________________________________________________________________________________

Proc BoxStyle() : GraphStyle
	PauseUpdate; Silent 1
	GetAxis/Q right
	if (V_flag == 1)
		ModifyGraph/Z mirror(left)=2
	endif
	GetAxis/Q top
	if (V_flag == 1)
		ModifyGraph/Z mirror(bottom)=2
	endif
	ModifyGraph noLabel=0
	ModifyGraph standoff=0
	ModifyGraph ZisZ=1
	ModifyGraph tick=0
	ModifyGraph notation(left)=1
	ModifyGraph axisOnTop=1
	ModifyGraph minor=1
	if ((GetKeyState(0) & 2^0) != 0)		// ctrl = paper style (no left axis notation)
		ModifyGraph tick(left)=3			// ticks off
		ModifyGraph noLabel(left)=1			// disable labels
	endif
	ResumeUpdate
EndMacro

Proc XASLabel() : GraphStyle
	Label/Z left "Intensity [arb.u.] \\E"
	Label/Z bottom "\u#2Photon energy [eV]"
EndMacro

Proc XESLabel() : GraphStyle
	Label/Z left "Intensity [arb.u.] \\E"
	Label/Z bottom "\u#2Emission Energy [eV]"
EndMacro

Proc ToFLabel() : GraphStyle
	Label/Z left "Intensity [arb.u.] \\E"
	Label/Z bottom "Flight time [\U]"
EndMacro

Proc PKELabel() : GraphStyle
	Label/Z left "\u#2Intensity [arb.u.] \\E"
	Label/Z bottom "\u#2"+SelectString((GetKeyState(0) & 2^0) != 0, "Kinetic", "Binding")+" Energy [eV]"		// BE label with ctrl
EndMacro

Proc LeftNoTick() : GraphStyle
	ModifyGraph tick(left)=3
	ModifyGraph noLabel(left)=1
	ModifyGraph margin(left)=30
EndMacro

Proc LeftEnabTick() : GraphStyle
	ModifyGraph tick(left)=0
	ModifyGraph noLabel(left)=0
	ModifyGraph margin(left)=0
EndMacro

//________________________________________________________________________________________________
//										Graph modify panel
// A small panel to modify multi-trace graphs - ver. 2020-10-10
//________________________________________________________________________________________________

Function TraceOffsetPanel()
	if (WinType("OffsetPanel"))																	// make sure that there is only one panel of this
		DoWindow/F OffsetPanel
		return 0
	endif
	if (ItemsinList(WinList("*", ";", "WIN:1")) == 0)
		Abort "No graphs available. Create a graph first."
	endif
	NewPanel/K=1/W=(450,60,620,540)/N=OffsetPanel as "Modify Graph"
	ModifyPanel /W=OffsetPanel ,fixedSize= 1 ,noEdit=1
	SetWindow OffsetPanel hook(UpdateHook)=OffsetPanelUpdateFunc
	
	SetDrawLayer UserBack
	DrawLine 10,190,160,190
	DrawLine 10,260,160,260
	DrawLine 10,410,160,410
	DrawLine 10,445,160,445
	
	SetVariable vSetYOffset		,pos={10,25}	,size={70,16}		,title="y:"
	SetVariable vScaleYOffset	,pos={10,65}	,size={70,16}		,title="y:"
	SetVariable vShiftYOffset	,pos={10,105}	,size={70,16}		,title="y:"
	SetVariable vSetXOffset		,pos={90,25}	,size={70,16}		,title="x:"
	SetVariable vScaleXOffset	,pos={90,65}	,size={70,16}		,title="x:"
	SetVariable vShiftXOffset	,pos={90,105}	,size={70,16}		,title="x:"
	
	TitleBox Settitle			,pos={10,10}	,size={150,13}		,title="Set offset per trace"				,frame=0
	TitleBox Scaletitle			,pos={10,50}	,size={150,13}		,title="Scale offset of all traces"			,frame=0
	TitleBox Shifttitle			,pos={10,90}	,size={150,13}		,title="Shift offset of all traces"			,frame=0
	Button ResetYButton			,pos={10,130}	,size={70,22}		,title="Y Reset"							,help={"Resets all Y offsets to 0."}
	Button ResetXButton			,pos={90,130}	,size={70,22}		,title="X Reset"							,help={"Resets all X offsets to 0."}
	Button ReverseButton		,pos={10,160}	,size={150,22}		,title="Reverse Sorting"					,help={"Reverses the order in which the traces are sorted in the graph."}
	
	PopupMenu Color				,pos={10,200}	,size={150,22}									,bodyWidth=150
	Button ColorButton			,pos={10,230}	,size={150,22}		,title="Colorize"							,help={"Colorize traces with currently selected color spectrum. To reverse spectrum press 'ctrl' or 'shift'."}
	PopupMenu AspectSelect		,pos={10,270}	,size={150,22}		,title="Aspect Ratio:"		,bodyWidth=70
	PopupMenu FormatSelect		,pos={10,295}	,size={150,22}		,title="Format:"			,bodyWidth=100
	
	Button CopyStyleButton		,pos={10,325}	,size={80,22}		,title="Copy Style"							,help={"Copies the style of the top graph."}
	CheckBox CopyAxes			,pos={100,328}	,size={50,22}		,title="w/ axes"			,value = 1		,help={"Copies the axis scaling too"}
	PopupMenu GrStyle			,pos={10,353}	,size={150,22}		,title="Style:"				,bodyWidth=115
	Button ApplyButton			,pos={10,380}	,size={70,22}		,title="\f01Apply"							,help={"Applies selected style to the top graph."}
	Button RevertButton			,pos={90,380}	,size={70,22}		,title="Revert"								,help={"Reverts to last style before Apply was pressed."}

	PopUpMenu ProfileSelect		,pos={10,420}	,size={150,20}		,title="Profile"			,bodywidth=115
	Button OpnSizePanelButton	,pos={10,455}	,size={150,22}		,title="Open Graph Size Panel..."			,help={"Opens a panel to set the graph's size and margin quickly"}
	
	SetVariable vSetYOffset		,proc=TraceOffsetExecute			,value=_NUM:0		,limits={0,inf,1}		,format="%g"
	SetVariable vScaleYOffset	,proc=TraceOffsetExecute			,value=_NUM:1		,limits={0,inf,0.1}		,format="%g"	,userdata="1"
	SetVariable vShiftYOffset	,proc=TraceOffsetExecute			,value=_NUM:0		,limits={-inf,inf,1}	,format="%g"
	SetVariable vSetXOffset		,proc=TraceOffsetExecute			,value=_NUM:0		,limits={-inf,inf,0.1}	,format="%g"
	SetVariable vScaleXOffset	,proc=TraceOffsetExecute			,value=_NUM:1		,limits={0,inf,0.1}		,format="%g"	,userdata="1"
	SetVariable vShiftXOffset	,proc=TraceOffsetExecute			,value=_NUM:0		,limits={-inf,inf,1}	,format="%g"
	PopupMenu Color				,mode=2								,value= "*COLORTABLEPOPNONAMES*"
	PopupMenu AspectSelect		,proc=ChangeAspectRatio	,mode=1		,value= "Free;1:1;-;4:3;3:2;16:9;-;2:3;3:4;4:5"
	PopupMenu FormatSelect		,proc=ChangeGraphFormat	,mode=1		,value= "Free Size;Paper Half;Paper Full;PPT 16:9 Half;PPT 16:9 Full;PPT 4:3 Half;PPT 4:3 Full;"
	PopupMenu GrStyle			,mode=1								,value= MacroList("*",";","KIND:1,SUBTYPE:GraphStyle")
	PopupMenu ProfileSelect		,proc=ProfileFromTraces	,mode=1		,value= TraceProfileWaveList()
	
	ModifyControlList ControlNameList("OffsetPanel",";","*Button") ,proc=TraceModifyExecute
	#if IgorVersion() >= 7.00
		ModifyControlList ControlNameList("OffsetPanel",";","v*") ,focusRing=0
		ModifyControlList "Color;GrStyle;ProfileSelect;" ,focusRing=0
	#endif
	FetchLatestScaleValues()																// set variable scales to top window values
End
//+++++++++++++++++++++++++++++++++ popup help functions +++++++++++++++++++++++++++++++++++++
Function/S TraceProfileWaveList()
	return "none;"+WaveList("*",";", "DIMS:1")
End
//################################ variable helper function ##################################
Function TraceOffsetExecute(SV) : SetVariableControl
	STRUCT WMSetVariableAction &SV
	if (SV.eventCode == 1 || SV.eventCode == 2)
		SV.blockReentry=1
		if (ItemsinList(WinList("*", ";", "WIN:1")) == 0)									// make sure there is a graph to work with
			return 0
		endif
		
		String Traces = TraceNameList("",";",1)
		Variable i, Items = ItemsInList(Traces)
		String xAxis, RemItem = ""
		for (i = 0; i < Items; i += 1)														// remove possible vertical traces
			xAxis = StringByKey("XAXIS",  TraceInfo("",StringFromList(Items - 1 - i,Traces),0))
			if (!StringMatch(xAxis,"bottom"))
				RemItem += StringFromList(Items - 1 - i,Traces)+";"
			endif
		endfor
		Traces	= RemoveFromList(RemItem,Traces)
		Items	= ItemsInList(Traces)
		
		Variable CurOffset	= 0, YnotX = 0
		Variable CurYShift	= str2num(StringFromList(1, ReplaceString("{", ReplaceString("}",StringByKey("offset(x)", TraceInfo("",StringFromList(Items - 1,Traces),0), "="),""),""), ","))
		Variable CurXShift	= str2num(StringFromList(0, ReplaceString("{", ReplaceString("}",StringByKey("offset(x)", TraceInfo("",StringFromList(Items - 1,Traces),0), "="),""),""), ","))
		StrSwitch(SV.ctrlName)
			case "vSetYOffset":
				YnotX = 1
			case "vSetXOffset":
				for (i = 0; i < Items; i += 1)
					if (YnotX)
						ModifyGraph offset($StringFromList(Items - 1 - i,Traces))={,i*SV.dval+CurYShift}
					else
						ModifyGraph offset($StringFromList(Items - 1 - i,Traces))={i*SV.dval+CurXShift,}
					endif
				endfor
			break
			case "vScaleYOffset":
				YnotX = 1
			case "vScaleXOffset":
				Variable PrevVal = str2num(SV.Userdata)										// get previous multiplier from control userdata
				PrevVal = PrevVal != 0 ? PrevVal : 1										// make sure multiplier is not zero
				SV.Userdata = num2str(SV.dval)												// write current multiplier
				for (i = 0; i < Items; i += 1)
					CurOffset = str2num(StringFromList(YnotX, ReplaceString("{", ReplaceString("}",StringByKey("offset(x)", TraceInfo("",StringFromList(Items - 1 - i,Traces),0), "="),""),""), ","))
					if (YnotX)
						ModifyGraph offset($StringFromList(Items - 1 - i,Traces))={,(CurOffset-CurYShift)*SV.dval/PrevVal+CurYShift}
					else
						ModifyGraph offset($StringFromList(Items - 1 - i,Traces))={(CurOffset-CurXShift)*SV.dval/PrevVal+CurXShift,}
					endif
				endfor
			break
			case "vShiftYOffset":
				YnotX = 1
			case "vShiftXOffset":
				for (i = 0; i < Items; i += 1)
					CurOffset = str2num(StringFromList(YnotX, ReplaceString("{", ReplaceString("}",StringByKey("offset(x)", TraceInfo("",StringFromList(Items - 1 - i,Traces),0), "="),""),""), ","))
					if (YnotX)
						ModifyGraph offset($StringFromList(Items - 1 - i,Traces))={,(CurOffset-CurYShift)+SV.dval}
					else
						ModifyGraph offset($StringFromList(Items - 1 - i,Traces))={(CurOffset-CurXShift)+SV.dval,}
					endif
				endfor
			break
		EndSwitch	
		AdjustProfileAxisScaling()
	endif
End
//+++++++++++++++++++++++++++++++ help function for Profile Axis +++++++++++++++++++++++++++++
Function AdjustProfileAxisScaling()
	Variable Scaling, MaxOffset = str2num(StringFromList(1, ReplaceString("{", ReplaceString("}",StringByKey("offset(x)", TraceInfo("","",0), "="),""),""), ","))
	if (FindListItem("Profileleft", AxisList("")) >= 0)
		DoUpdate; GetAxis/Q left
		Scaling	= (MaxOffset-V_min)/(V_Max-V_min)											// calculate the ratio between highest offset and axis height
		Scaling	= Limit(Scaling, 0.1, 1)													// prevent scaling to useless values
		ModifyGraph axisEnab(Profileleft)={0,Scaling}
	endif
End
//################################## button helper function ##################################
Function TraceModifyExecute(SB) : ButtonControl
	STRUCT WMButtonAction &SB
	if (SB.eventCode == 2)
		SB.blockReentry=1
		if (ItemsinList(WinList("*", ";", "WIN:1")) == 0)
			return 0
		endif
		String TopGraph = WinName(0, 1), cmd = "", Style = ""
		strswitch(SB.ctrlName)
			case "OpnSizePanelButton":
				MakeGraphSizePanel()
				break
			case "ColorButton":
				ControlInfo/W=$(SB.win) Color
				String Table = S_value
				ColorSpectrumFromTable(GetKeyState(0)>0, table)
				break
			case "ReverseButton":
				String Traces	= TraceNameList("",";",1)
				Variable Items	= ItemsInList(Traces)

				Variable i
				String xAxis, RemItem = ""
				for (i = 0; i < Items; i += 1)												// remove possible vertical traces
					xAxis = StringByKey("XAXIS",  TraceInfo("",StringFromList(Items - 1 - i,Traces),0))
					if (!StringMatch(xAxis,"bottom"))
						RemItem += StringFromList(Items - 1 - i,Traces)+";"
					endif
				endfor
				Traces	= RemoveFromList(RemItem,Traces)
				Items	= ItemsInList(Traces)
		
				String reversed=""
				for (i = 0; i < Items; i += 1)												// reverse trace list
					reversed += StringFromList(Items - 1 - i,Traces) + ";"
				endfor
				Traces = RemoveEnding(ReplaceString(";", reversed, ","), ",")
				Execute "ReorderTraces _back_,{"+traces+"}"									// execute reverse operation
			case "ResetYButton":															// no break here... reverse button also resets trace offsets
				ModifyGraph offset={,0}//,muloffset={,0}
				SetVariable vSetYOffset		,win=OffsetPanel	,value=_NUM:0
				SetVariable vScaleYOffset	,win=OffsetPanel	,value=_NUM:1	,userdata="1"
				SetVariable vShiftYOffset	,win=OffsetPanel	,value=_NUM:0
				AdjustProfileAxisScaling()													// resizes the profile axis
				break
			case "ResetXButton":
				ModifyGraph offset={0,}//,muloffset={0,}
				SetVariable vSetXOffset		,win=OffsetPanel	,value=_NUM:0
				SetVariable vScaleXOffset	,win=OffsetPanel	,value=_NUM:1	,userdata="1"
				SetVariable vShiftXOffset	,win=OffsetPanel	,value=_NUM:0
				AdjustProfileAxisScaling()													// resizes the profile axis
				break
			case "ApplyButton":																// applies selected graph style
				ControlInfo/W=$(SB.win) GrStyle
				Style = S_value
				String LastStyle = WinRecreation(TopGraph,0)
				if (StringMatch(Style,"Copied Style"))
					Style = GetUserData("OffsetPanel", "", "CopyStyle")
					Variable lines = itemsinlist(Style)
					for (i=0;i<lines;i+=1)
					    Execute/Q StringFromList(i,Style)
					endfor
				else
					Execute/Q Style+"()"
				endif
				SetWindow $TopGraph, userdata(LastStyle)=LastStyle							// save into graphs userdata
				break
			case "RevertButton":															// copies top-graph's style
				Style = GetUserData(TopGraph, "", "LastStyle")								// check for existence of backup style
				if (strlen(Style) > 0)
					KillWindow $TopGraph
					String/G ExeCmd = Style
					Execute/P/Q "Execute ExeCmd"
					Execute/P/Q "KillStrings/Z ExeCmd"
				endif
				break
			case "CopyStyleButton":															// copies top-graph's style
				Style = WinRecreation(TopGraph,1)
				Style = GrepList(Style, "^\t(DefineGuide|ControlBar|SetWindow|Cursor|ListBox|CheckBox|PopupMenu|ValDisplay|SetVariable|Button|NewPanel|ModifyPanel|RenameWindow|SetActiveSubwindow|ShowTools)", 1, "\r")// remove control items
				Style = GrepList(Style, "^\t(ModifyGraph/Z rgb|ModifyGraph/Z muloffset|ModifyGraph/Z offset|ShowInfo)", 1, "\r")
				ControlInfo/W=OffsetPanel CopyAxes
				if (!V_Value)
					Style = GrepList(Style, "^\t(SetAxis)", 1, "\r")
				endif
				Style = RemoveListItem(0,Style, "\r")										// remove "proc" line
				Style = RemoveListItem(0,Style, "\r")										// remove "pauseupdate" line
				Style = RemoveListItem(ItemsInList(Style,"\r") - 1,Style, "\r")				// remove "endmacro" line
				Style = ReplaceString("\t", Style, "")
				Style = ReplaceString("\r", Style, ";")
				
				SetWindow OffsetPanel, userdata(CopyStyle)=Style							// save into panels userdata
				PopupMenu GrStyle	,win=OffsetPanel	 ,value= "Copied Style;"+MacroList("*",";","KIND:1,SUBTYPE:GraphStyle") ,mode= 1
				break
		endswitch
	endif
End
//################################# popup helper function ####################################
Function ProfileFromTraces (ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr	
	Variable popNum

	SetActiveSubwindow $WinName(0,1)														// make sure to do stuff in the graph window
	if (FindListItem("Profileleft", AxisList(WinName(0,1))) != -1)							// see if the additional Profile axis is attached
		RemoveFromGraph $("AddTraceProfile")												// remove the old trace (if present)
		ModifyGraph axisEnab(bottom)={0,1}													// reset the axes and remove the drawn line from previous selections
	endif

	StrSwitch(popStr)
		case "none":																		// do nothing
			break
		default:
			Wave inwave = $popStr
			ModifyGraph axisEnab(bottom)={0,0.72}											// make the bottom axis shorter
			AppendToGraph/C=(0,15872,65280)/B=Profilebottom/L=Profileleft/VERT inwave/TN=AddTraceProfile	// append the Profile trace
			ModifyGraph mirror(Profilebottom)=2
			ModifyGraph axisEnab(Profilebottom)={0.76,1}	
			ModifyGraph freePos(Profilebottom)=0 ,freePos(Profileleft)={0.76,kwFraction}	// move the new axes to the right position
	
			ModifyGraph tkLblRot(Profileleft)=90											// rotate the tick labels
			ModifyGraph minor(Profileleft)=1, tick(Profileleft)=2							// minor ticks
			ModifyGraph standoff=0															// axes are aligned, having no 1 pixel standoff
			ModifyGraph ZisZ(Profilebottom)=1,	ZisZ(Profileleft)=1							// zero is 0 (not 0.00)
			ModifyGraph notation(Profilebottom)=1											// scientific notation (10^4 instead of 10*10^3)
			ModifyGraph tlOffset(Profileleft)=-2											// move the labels in a bit

			AdjustProfileAxisScaling()														// resizes the profile axis
		break
	EndSwitch
End
//################################## popup helper function ###################################
Function ChangeAspectRatio (ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr	
	Variable popNum
	String HeightMode = ""
	StrSwitch (popStr)
		case "Free":
			HeightMode = "0"
		break
		case "1:1":
			HeightMode = "{Aspect,1}"
		break
		case "4:3":
			HeightMode = "{Aspect,3/4}"
		break
		case "3:2":
			HeightMode = "{Aspect,2/3}"
		break
		case "16:9":
			HeightMode = "{Aspect,9/16}"
		break
		case "2:3":
			HeightMode = "{Aspect,3/2}"
		break
		case "3:4":
			HeightMode = "{Aspect,4/3}"
		break
		case "4:5":
			HeightMode = "{Aspect,5/4}"
		break
	EndSwitch
	if (strlen(HeightMode) > 0)
		Execute/Q "ModifyGraph/Z height="+HeightMode
	endif
End
//################################## popup helper function ###################################
Function ChangeGraphFormat (ctrlName,popNum,popStr) : PopupMenuControl
	String ctrlName, popStr	
	Variable popNum
	
	if (StringMatch(popStr,"Free Size"))
		ModifyGraph/Z width=0, height=0, margin=0
		ModifyGraph/Z gfSize=0, fSize=0, lsize=1
		ModifyGraph/Z expand=0
		return 0
	endif
	
	GetAxis/Q bottom
	Variable bottomOn = !V_Flag
	GetAxis/Q left
	Variable leftOn = !V_Flag
	GetAxis/Q right
	Variable rightOn = !V_Flag
	GetAxis/Q top
	Variable topOn = !V_Flag
	
	if (StringMatch(popStr, "Paper*")) 
		ModifyGraph/Z gfSize=8, fSize=8
		ModifyGraph/Z expand=1.25
		ModifyGraph/Z lsize=1.5
		
		ModifyGraph/Z margin(left)	= (11 + 22*leftOn + 10*(bottomOn || topOn && !leftOn))
		ModifyGraph/Z margin(bottom)= (11 + 22*bottomOn)
		ModifyGraph/Z margin(top)	= (11 + 22*topOn)
		ModifyGraph/Z margin(right)	= (11 + 22*rightOn + 10*(bottomOn || topOn && !rightOn))
		
		if (StringMatch(popStr, "*Half"))
			ModifyGraph/Z width=(180 - 12*(leftOn && rightOn))
			ModifyGraph/Z height=(172 - 22*(bottomOn && topOn))
		elseif (StringMatch(popStr, "*Full"))
			ModifyGraph/Z width=(450 - 12*(leftOn && rightOn))
			ModifyGraph/Z height=(208 - 22*(bottomOn && topOn))
		endif
	endif
	
	if (StringMatch(popStr, "PPT*")) 
		ModifyGraph/Z gfSize=16, fSize=12
		ModifyGraph/Z expand=0.75
		ModifyGraph/Z lsize=1.5

		ModifyGraph/Z margin(left)	= (18 + 37*leftOn + 12*(bottomOn || topOn && !leftOn))
		ModifyGraph/Z margin(bottom)= (18 + 37*bottomOn)
		ModifyGraph/Z margin(top)	= (18 + 37*topOn)
		ModifyGraph/Z margin(right)	= (18 + 37*rightOn + 12*(bottomOn || topOn && !rightOn))
		
		if (StringMatch(popStr, "*16:9*"))
			ModifyGraph/Z height=(360 - 37*(bottomOn && topOn))
			if (StringMatch(popStr, "*Half"))
				ModifyGraph/Z width=(380 - 25*(leftOn && rightOn))
			elseif (StringMatch(popStr, "*Full"))
				ModifyGraph/Z width=(800 - 25*(leftOn && rightOn))
			endif
		elseif (StringMatch(popStr, "*4:3*"))
			ModifyGraph/Z height=(350 - 37*(bottomOn && topOn))
			if (StringMatch(popStr, "*Half"))
				ModifyGraph/Z width=(290 - 25*(leftOn && rightOn))
			elseif (StringMatch(popStr, "*Full"))
				ModifyGraph/Z width=(600 - 25*(leftOn && rightOn))
			endif
		endif
	endif
	
	DoUpdate
	ModifyGraph/Z height=0
End

//################################ panel update helper function ##################################
Function OffsetPanelUpdateFunc(s)
	STRUCT WMWinHookStruct &s
	If (s.eventCode == 0)	// Activate
		FetchLatestScaleValues()
		return 1
	endif
	return 0
End
//################################################################################################
Function FetchLatestScaleValues()
	if (ItemsinList(WinList("*", ";", "WIN:1")) == 0)
		return 0
	endif
	String Traces	= TraceNameList("",";",1)
	Variable Items	= ItemsInList(Traces)
		
	Variable i
	String xAxis, RemItem = ""
	for (i = 0; i < Items; i += 1)																// remove possible vertical traces
		xAxis = StringByKey("XAXIS",  TraceInfo("",StringFromList(Items - 1 - i,Traces),0))
		if (!StringMatch(xAxis,"bottom"))
			RemItem += StringFromList(Items - 1 - i,Traces)+";"
		endif
	endfor
	Traces	= RemoveFromList(RemItem,Traces)
	Items	= ItemsInList(Traces)

	if (Items > 1)																				// calculate shift and offset from first two traces
		Variable xOff = 0, yOff = 0, xShift = 0,  yShift = 0
		sscanf StringByKey("offset(x)", TraceInfo("",StringFromList(Items-2,Traces),0), "="), "{%f,%f}", xOff, yOff
		sscanf StringByKey("offset(x)", TraceInfo("",StringFromList(Items-1,Traces),0), "="), "{%f,%f}", xShift, yShift	
	endif

	GetAxis/Q left
	Variable YAxisMax = V_max
	GetAxis/Q bottom
	Variable XAxisRange = abs(V_max - V_min)
	
	String str	// rounding
	sprintf str, "%.*g\r", 1, YAxisMax;		YAxisMax = str2num(str)
	sprintf str, "%.*g\r", 1, XAxisRange;	XAxisRange = str2num(str)

	SetVariable vSetYOffset		,win=OffsetPanel	,value=_NUM:yOff-yShift	,limits={0,inf,YAxisMax*0.005}
	SetVariable vShiftYOffset	,win=OffsetPanel	,value=_NUM:yShift		,limits={-inf,inf,YAxisMax*0.005}
	SetVariable vScaleYOffset	,win=OffsetPanel	,value=_NUM:1			,userdata="1"
	SetVariable vSetXOffset		,win=OffsetPanel	,value=_NUM:xOff-xShift	,limits={-inf,inf,XAxisRange*0.005}
	SetVariable vShiftXOffset	,win=OffsetPanel	,value=_NUM:xShift		,limits={-inf,inf,XAxisRange*0.05}
	SetVariable vScaleXOffset	,win=OffsetPanel	,value=_NUM:1			,userdata="1"
	//--------------------------------------------------------------------------------------------
	PopupMenu ProfileSelect,win=OffsetPanel		,mode=1											// update the selected profile or select none
	if (FindListItem("Profileleft", AxisList("")) >= 0)
		Wave profile = TraceNameToWaveRef("", "AddTraceProfile")
		PopupMenu ProfileSelect,win=OffsetPanel	,popmatch=nameofwave(profile)
	endif
	
	Variable StartPos = 0, EndPos = 0, Aspect = 0, PopAspect = 0
	String Setting = ListMatch(WinRecreation(WinName(0,1),1), "*height={Aspect,*","\r")
	if (strlen(Setting) > 0)																	// extract aspect setting
		StartPos	= StrSearch(Setting,"Aspect,",StartPos) + strlen("Aspect,")
		EndPos		= StrSearch(Setting,"}",StartPos) - strlen("}")
		Aspect		= str2num(Setting[StartPos,EndPos])
	endif
	
	String PopList = "free;1:1;4:3;3:2;16:9;2:3;3:4;4:5"										// popup match list
	PopAspect += 1 * (Aspect == 1)
	PopAspect += 2 * (Aspect == 0.75)
	PopAspect += 3 * (Aspect < 0.6668 && Aspect > 0.6666)
	PopAspect += 4 * (Aspect < 0.563 && Aspect > 0.562)
	PopAspect += 5 * (Aspect == 1.5)
	PopAspect += 6 * (Aspect < 1.334 && Aspect > 1.332)
	PopAspect += 7 * (Aspect == 1.25)
	PopupMenu AspectSelect ,win=OffsetPanel ,popmatch=StringFromList(PopAspect,PopList)
End

//################################################################################################

Function ColorSpectrumFromTable(rev, table)
	Variable rev
	String table
	
	String Traces	= TraceNameList("",";",1)
	Variable Items	= ItemsInList(Traces)
	if (Items == 0)
		Traces	= ImageNameList("",";")
		Items	= ItemsInList(Traces)
		if (Items == 0)
			return 0
		endif
		ModifyImage $StringFromList(0,Traces) ctab={,,$table,rev}
		return 0
	endif
	ColorTab2Wave $table
	Wave M_colors
	Variable i, traceindex
	for (i = 0; i < Items; i += 1)
		Variable row = (i/(Items-1))*(DimSize(M_Colors, 0)-1)
		traceindex = ( rev == 0 ? i : Items-1-i)
		Variable red = M_Colors[row][0], green = M_Colors[row][1], blue = M_Colors[row][2]
		ModifyGraph/Z rgb[traceindex] = (red, green, blue)
	endfor
	KillWaves/Z M_colors
End

//________________________________________________________________________________________________
//										Graph size panel
// A small panel to set the graph's size and margins - ver. 2020-10-10
//________________________________________________________________________________________________

Function MakeGraphSizePanel()
	if (WinType("GraphSizePanel"))
		DoWindow/F GraphSizePanel
		return 0
	endif
	if (ItemsinList(WinList("*", ";", "WIN:1")) == 0)
		Abort "No graphs available. Create a graph first."
	endif
	NewPanel/K=1/W=(500,60,810,285)/N=GraphSizePanel as "Set Graph Size"
	ModifyPanel /W=GraphSizePanel ,fixedSize= 1 ,noEdit=1
	SetWindow GraphSizePanel hook(UpdateHook)=SizePanelUpdateFunc
	
	SetDrawLayer UserBack
	SetDrawEnv linethick= 2,fillpat= 0
	DrawRect 20,10,287,195
	SetDrawEnv linethick= 2,dash= 3,fillpat= 0
	DrawRect 50,40,255,165
	SetDrawEnv linethick= 2,linefgc= (0,0,65535),arrow= 3
	DrawLine 53,70,252,70
	SetDrawEnv linethick= 2,linefgc= (0,0,65535),arrow= 3
	DrawLine 205,43,205,163
	SetDrawEnv linethick= 2,linefgc= (1,39321,19939),arrow= 3
	DrawLine 28,135,280,135
	SetDrawEnv linethick= 2,linefgc= (1,39321,19939),arrow= 3
	DrawLine 105,13,105,193
	
	SetVariable VLeftMargin		,pos={5		,95}	,value=_NUM:0
	SetVariable VBottomMargin	,pos={127	,170}	,value=_NUM:0
	SetVariable VRightMargin	,pos={245	,95}	,value=_NUM:0
	SetVariable VTopMargin		,pos={127	,15}	,value=_NUM:0
	SetVariable VGraphHeight	,pos={75	,95}	,value=_NUM:0
	SetVariable VPlotWidth		,pos={125	,60}	,value=_NUM:0
	SetVariable VGraphWidth		,pos={125	,125}	,value=_NUM:0
	SetVariable VPlotHeight		,pos={175	,95}	,value=_NUM:0
	SetVariable VMagnification	,pos={215	,170}	,value=_NUM:0
	SetVariable VGraphFont		,pos={37	,170}	,value=_NUM:0
	
	ModifyControlList ControlNameList("GraphSizePanel",";","*")	,limits={0,2000,1}	,size={60.00,18.00} ,bodyWidth=60 ,proc=SizeVarControl
	
	SetVariable VMagnification	,title="exp:"	,limits={0,8,0} 	,bodyWidth=40	,format="%g x"
	SetVariable VGraphFont		,title="font:"	,limits={0,100,0}	,bodyWidth=40
	
	TitleBox GraphTitle			,pos={25	,15}	,size={60,23}		,title="none"				,frame=0
	TitleBox SwapTitle			,pos={140	,80}	,size={60,23}		,title="\JCswap:"			,frame=0
	CheckBox SwapAxes			,pos={148	,98}	,size={23,23}		,title=""					,proc=SwapGraphControl	,value=0
	CheckBox KeepAutoMode		,pos={12	,203}	,size={23,23}		,title="keep auto mode"								,value=1	,help={"Automatically resets the mode to Auto after changing the size."}
	PopupMenu SizeUnit			,pos={190	,15}	,size={70,20}		,title="unit: "				,proc=SizePopControl
	Button CopyAllButton		,pos={120	,200}	,size={85,20}		,title="copy values"		,proc=CopyPasteControl
	Button PasteAllButton		,pos={215	,200}	,size={85,20}		,title="paste copied"		,proc=CopyPasteControl
	
	PopupMenu SizeUnit			,mode=1				,popvalue="points"	,value= #"\"points;inches;cm\""
	
	FetchLatestGraphSizeValues()
End
//################################################################################################
Function SizePopControl(s) : PopupMenuControl
	STRUCT WMPopupAction &s
	if (s.eventCode != 2)
		return 0
	endif
	String CtrlList = ControlNameList("GraphSizePanel",";","V*")
	CtrlList = RemoveFromList("VMagnification;VGraphFont;",CtrlList)
	Variable Increment = 1
	if (s.popNum == 2)		// in inches
		Increment = 0.02
	elseif (s.popNum == 3)	// in cm
		Increment = 0.05
	endif
	ModifyControlList CtrlList ,win=GraphSizePanel	,limits={0,2000,Increment}
	FetchLatestGraphSizeValues()
End
//################################################################################################
Function SwapGraphControl(cb) : CheckBoxControl
	STRUCT WMCheckboxAction& cb
	if (cb.eventCode != 2)
		return 0
	endif
	if (ItemsinList(WinList("*", ";", "WIN:1")) == 0)
		return 0
	endif
	String gName = StringFromList(0,WinList("*", ";", "WIN:1"))
	ModifyGraph/W=$gName swapXY=cb.checked
	DoUpdate/W=$gName
	FetchLatestGraphSizeValues()
End
//################################################################################################
Function CopyPasteControl(s) : ButtonControl
	STRUCT WMButtonAction &s
	if (s.eventCode != 2)
		return 0
	endif
	
	String CtrlList = "", CtrlName = "", ValList = "", Others
	Variable i, Value = 0
	StrSwitch(s.ctrlName)
		case "CopyAllButton":
			CtrlList = ControlNameList("GraphSizePanel", ";", "V*")
			CtrlList = RemoveFromList("VGraphWidth;VGraphHeight;",CtrlList)
			for (i = 0; i < ItemsInList(CtrlList); i += 1)
				CtrlName = StringFromList(i,CtrlList)
				ControlInfo/W=GraphSizePanel $CtrlName
				ValList += num2str(V_Value) + ";"
			endfor
			
			Others = ""
			ControlInfo/W=GraphSizePanel SwapAxes
			Others +=  "SwapAxes:" + num2str(V_Value) + ";"
			ControlInfo/W=GraphSizePanel SizeUnit
			Others +=  "SizeUnit:" + num2str(V_Value) + ";"
			
			SetWindow GraphSizePanel ,userdata(CtrlSet)=CtrlList
			SetWindow GraphSizePanel ,userdata(ValSet)=ValList
			SetWindow GraphSizePanel ,userdata(OtherSet)=Others
		break
		case "PasteAllButton":
			CtrlList	= GetUserData("GraphSizePanel", "", "CtrlSet")
			ValList		= GetUserData("GraphSizePanel", "", "ValSet")
			Others		= GetUserData("GraphSizePanel", "", "OtherSet")
			if (strlen(CtrlList) == 0)
				break
			endif
			CheckBox SwapAxes  ,win=GraphSizePanel	,value=str2num(StringByKey("SwapAxes", Others))
			PopupMenu SizeUnit ,win=GraphSizePanel	,mode=str2num(StringByKey("SizeUnit", Others))
			for (i = 0; i < ItemsInList(CtrlList); i += 1)
				CtrlName = StringFromList(i,CtrlList)
				Value = str2num(StringFromList(i,ValList))
				SetVariable $CtrlName ,win=GraphSizePanel ,value=_NUM:Value
				ExecuteVarChange(CtrlName, Value)
			endfor
		break
	EndSwitch
		
	FetchLatestGraphSizeValues()
End
//################################################################################################
Function SizeVarControl(s) : SetVariableControl
	struct WMSetVariableAction &s
	if (s.eventCode != 1 && s.eventCode != 2)
		return 0
	endif
	ExecuteVarChange(s.ctrlName, s.dval)
End
//################################################################################################
Function ExecuteVarChange(ctrlName, val)
	String ctrlName
	Variable val
	if (ItemsinList(WinList("*", ";", "WIN:1")) == 0)
		return 0
	endif
	String gName = StringFromList(0,WinList("*", ";", "WIN:1"))
	
	Variable Modifier = 1
	ControlInfo/W=GraphSizePanel SizeUnit
	if (V_Value == 2)		// in inches
		Modifier = 72
	elseif (V_Value == 3)	// in cm
		Modifier = 72/2.54
	endif
	
	ControlInfo/W=GraphSizePanel KeepAutoMode
	Variable Auto = V_Value
	
	StrSwitch (ctrlName)
		case "VLeftMargin":
			ModifyGraph/W=$gName margin(left) = val*Modifier
		break
		case "VBottomMargin":
			ModifyGraph/W=$gName margin(bottom) = val*Modifier
		break
		case "VRightMargin":
			ModifyGraph/W=$gName margin(right) = val*Modifier
		break
		case "VTopMargin":
			ModifyGraph/W=$gName margin(top) = val*Modifier
		break
		case "VGraphHeight":
			ControlInfo/W=GraphSizePanel VTopMargin
			Variable mTop = V_Value
			ControlInfo/W=GraphSizePanel VBottomMargin
			Variable mBot = V_Value
			Variable pHeight = val-mTop-mBot
			if (val == 0)
				ModifyGraph/W=$gName margin(top) = 0
				ModifyGraph/W=$gName margin(bottom) = 0
				ModifyGraph/W=$gName height = 0
			elseif (pHeight > 0)
				ModifyGraph/W=$gName height = pHeight*Modifier
				if (Auto)
					DoUpdate/W=$gName
					ModifyGraph/W=$gName height = 0
				endif
			endif
		break
		case "VPlotWidth":
			ModifyGraph/W=$gName width = val*Modifier
			if (Auto)
				DoUpdate/W=$gName
				ModifyGraph/W=$gName width = 0
			endif
		break
		case "VGraphWidth":
			ControlInfo/W=GraphSizePanel VLeftMargin
			Variable mLeft = V_Value
			ControlInfo/W=GraphSizePanel VRightMargin
			Variable mRight = V_Value
			Variable pWidth = val-mLeft-mRight
			if (val == 0)
				ModifyGraph/W=$gName margin(left) = 0
				ModifyGraph/W=$gName margin(right) = 0
				ModifyGraph/W=$gName width = 0
			elseif (pWidth > 0)
				ModifyGraph/W=$gName width = pWidth*Modifier
				if (Auto)
					DoUpdate/W=$gName
					ModifyGraph/W=$gName width = 0
				endif
			endif
		break
		case "VPlotHeight":
			ModifyGraph/W=$gName height = val*Modifier
			if (Auto)
				DoUpdate/W=$gName
				ModifyGraph/W=$gName height = 0
			endif
		break
		case "VMagnification":
			Variable Mag = val == 1 ? 0 : val
			ModifyGraph/W=$gName expand = Mag
		break
		case "VGraphFont":
			ModifyGraph/W=$gName gfSize = val
		break
	EndSwitch
	
	DoUpdate/W=$gName
	FetchLatestGraphSizeValues()
end
//################################ panel update helper function ##################################
Function SizePanelUpdateFunc(s)
	STRUCT WMWinHookStruct &s
	If (s.eventCode == 0)
		FetchLatestGraphSizeValues()
		return 1
	endif
	return 0
End
//################################################################################################
Function FetchLatestGraphSizeValues()
	if (ItemsinList(WinList("*", ";", "WIN:1")) == 0)
		TitleBox GraphTitle ,win=GraphSizePanel ,title="No Graph"
		return 0
	endif
	String gName = StringFromList(0,WinList("*", ";", "WIN:1"))
	
	Variable Modifier = 1
	ControlInfo/W=GraphSizePanel SizeUnit
	if (V_Value == 2)		// in inches
		Modifier = 1/72
	elseif (V_Value == 3)	// in cm
		Modifier = 2.54/72
	endif
	
	// values from getwindow (sometimes off by a fraction?)
	GetWindow $gName gsize
	//print V_left, V_right, V_top, V_bottom		// debug
  	Variable gLeft	= V_left
  	Variable gTop	= V_top
  	Variable gWidth	= V_right-V_left	
  	Variable gHeight= V_bottom-V_top	
	GetWindow $gName psize
	//print V_left, V_right, V_top, V_bottom		// debug
	Variable mLeft	= round(V_left - gLeft)
	Variable mTop	= round(V_top - gTop)
	Variable mRight	= round(gWidth - V_right)
	Variable mBottom= round(gHeight - V_bottom)
	Variable pWidth	= round(V_right - V_left)
	Variable pHeight= round(V_bottom - V_top)
	gHeight	= round(gHeight)
	gWidth	= round(gWidth)
	
	// if available, replace with values from recreation macro (these are exact)
	String GraphSettings = GrepList(WinRecreation(gName, 0), "^\t(ModifyGraph)", 0 , "\r")
	GraphSettings = GrepList(GraphSettings, "(?i)(height|width|margin|gfsize|swapXY)", 0 , "\r")
	GraphSettings = ReplaceString("\tModifyGraph ",GraphSettings,"")
	GraphSettings = ReplaceString("\r",GraphSettings,",")

	Variable SetmLeft	= str2num(StringByKey("margin(left)", GraphSettings, "=", ","))
	Variable SetmTop	= str2num(StringByKey("margin(top)", GraphSettings, "=", ","))
	Variable SetmRight	= str2num(StringByKey("margin(right)", GraphSettings, "=", ","))
	Variable SetmBottom	= str2num(StringByKey("margin(bottom)", GraphSettings, "=", ","))
	Variable SetpWidth	= str2num(StringByKey("width", GraphSettings, "=", ","))
	Variable SetpHeight	= str2num(StringByKey("height", GraphSettings, "=", ","))
	Variable SetgHeight	= SetpHeight + SetmTop + SetmBottom
	Variable SetgWidth	= SetpWidth + SetmRight + SetmLeft

	Variable SetSwap	= str2num(StringByKey("swapXY", GraphSettings, "=", ","))
	Variable SetgFont	= str2num(StringByKey("gfSize", GraphSettings, "=", ","))
	Variable gFont		= numtype(SetgFont) == 0 ? SetgFont : 10
	setSwap	= numtype(setSwap) == 0 ? setSwap : 0
	
	Variable gMag = abs(str2num(StringByKey("expand", GraphSettings, "=", ",")))
	gMag	= numtype(gMag) == 0 ? gMag : 1
	#if IgorVersion() >= 7.00
		GetWindow $gName expand
		gMag = V_Value == 0 ? 1 : V_Value
	#endif

	Variable temp = 0
	if (setSwap)		// swap all values
		temp 		= SetmLeft
		SetmLeft	= SetmBottom
		SetmBottom	= temp
		temp		= SetmRight
		SetmRight	= SetmTop
		SetmTop		= temp
		temp		= SetgHeight
		SetgHeight	= SetgWidth
		SetgWidth	= temp
		temp		= SetpWidth
		SetpWidth	= SetpHeight
		SetpHeight	= temp
	endif
	
	mLeft	= numtype(SetmLeft) == 0 ? SetmLeft : mLeft
	mBottom	= numtype(SetmBottom) == 0 ? SetmBottom : mBottom
	mRight	= numtype(SetmRight) == 0 ? SetmRight : mRight
	mTop	= numtype(SetmTop) == 0 ? SetmTop : mTop
	pWidth	= numtype(SetpWidth) == 0 ? SetpWidth : pWidth
	pHeight	= numtype(SetpHeight) == 0 ? SetpHeight : pHeight
	gHeight	= numtype(SetgHeight) == 0 ? SetgHeight : gHeight
	gWidth	= numtype(SetgWidth) == 0 ? SetgWidth : gWidth
	
	
	// recalculate into inch or cm if needed
	if (Modifier != 1)
		mLeft	= round((mLeft*Modifier)*100)/100
		mBottom	= round((mBottom*Modifier)*100)/100
		mRight	= round((mRight*Modifier)*100)/100
		mTop	= round((mTop*Modifier)*100)/100
		gHeight	= round((gHeight*Modifier)*100)/100
		pWidth	= round((pWidth*Modifier)*100)/100
		gWidth	= round((gWidth*Modifier)*100)/100
		pHeight	= round((pHeight*Modifier)*100)/100
	endif

	TitleBox GraphTitle			,win=GraphSizePanel	,title=gName
	CheckBox SwapAxes			,win=GraphSizePanel	,value=setSwap
	SetVariable VLeftMargin		,win=GraphSizePanel	,value=_NUM:mLeft	,fstyle=(numtype(SetmLeft)==0)
	SetVariable VBottomMargin	,win=GraphSizePanel	,value=_NUM:mBottom	,fstyle=(numtype(SetmBottom)==0)
	SetVariable VRightMargin	,win=GraphSizePanel	,value=_NUM:mRight	,fstyle=(numtype(SetmRight)==0)
	SetVariable VTopMargin		,win=GraphSizePanel	,value=_NUM:mTop	,fstyle=(numtype(SetmTop)==0)
	SetVariable VGraphHeight	,win=GraphSizePanel	,value=_NUM:gHeight	,fstyle=(numtype(SetgHeight)==0)
	SetVariable VPlotWidth		,win=GraphSizePanel	,value=_NUM:pWidth	,fstyle=(numtype(SetpWidth)==0)
	SetVariable VGraphWidth		,win=GraphSizePanel	,value=_NUM:gWidth	,fstyle=(numtype(SetgWidth)==0)
	SetVariable VPlotHeight		,win=GraphSizePanel	,value=_NUM:pHeight	,fstyle=(numtype(SetpHeight)==0)
	SetVariable VMagnification	,win=GraphSizePanel	,value=_NUM:gMag
	SetVariable VGraphFont		,win=GraphSizePanel	,value=_NUM:gFont	,fstyle=(numtype(SetgFont)==0)
End